/*
 * Galactic Bloodshed, copyright (c) 1989 by Robert P. Chansky,
 * smq@ucscb.ucsc.edu 
 *
 * orbit.c -- display orbits of planets (graphic representation) 
 *
 * OPTIONS
 * -p : If this option is set, ``orbit'' will not display planet names. 
 *
 * -S : Do not display star names. 
 *
 * -s : Do not display ships. 
 *
 * -(number) : Do not display that #'d ship or planet (in case it obstructs
 * the view of another object) 
 */

#include "GB_copyright.h"
#define NAMESIZE 18

#define SCALE 100.0
#define S_X  (stdscr->_maxx - NAMESIZE)
#define S_Y  (stdscr->_maxy - 1)
#define Midx (0.5*(S_X - NAMESIZE)/SCALE)
#define Midy (0.5*S_Y /SCALE)

#include <curses.h>
#include <stdio.h>
#include "arrays.h"
#include "client.h"
extern char	*index(), *malloc();
extern hugestr	current_output;
extern int	have_inv;

float           Lastx, Lasty, Zoom;

plot_orbit(t)
	char           *t;
{
	int             x, y, array;
	int             stand1, stand2;
	char           *u, *v;
	char            temp[1024], symbol;
	int		more_data = 1;
	string		blob;


	clear();

	u = t + 1;
	while (more_data) {
	    while ((v = index(u, ';')) != NULL) {
		*v = 0;
		sscanf(u, "%d %d %d %d %1s %d %s", &stand1, &x, &y, &array,
		       &symbol, &stand2, temp);
		x = (int) x * Midx;
		y = (int) y * Midy;
		if (x <= S_X && y <= S_Y) {

			if (symbol == 'm') {	/* space mirror */
				DispArray(x, y, 9, 5, Mirror[array], 1.0);
			} else if (symbol == '*' && array > 0) {    /* nova */
				DispArray(x, y, 11, 7, Novae[array - 1], 1.0);
			}
			if (stand1 * have_inv)
				standout();
			mvaddch(y, x, symbol);
			if (stand1 * have_inv)
				standend();
			if (symbol == '@' || symbol == '*') {
				addstr(" ");
			}
			if (stand2 * have_inv)
				standout();
			addstr(temp);
			if (stand2 * have_inv)
				standend();
		}
		u = v + 1;
	    }

	    move(S_Y - 1, 0);
	    if (strlen(u) == 0) {
		refresh();
		return;
	    }
	    if (index(u, '\n') != NULL) {
		print_with_wrap(u);
		more_data = 0;
	    } else {
		receive(blob);
		strcpy(current_output, u);
		strcat(current_output, blob);
		u = current_output;
	    }
	}

	refresh();
	return;
}

/*
 * display array on screen.  at the moment magnification is not implemented. 
 */
DispArray(x, y, maxx, maxy, array, mag)
	int             x, y;
	int             maxx, maxy;
	char           *array[];
	float           mag;
{
	register int    x2, y2, curx, cury;

	for (cury = y - maxy / 2, y2 = 0; y2 < maxy; cury++) {
		if (cury >= 0 && cury <= S_Y) {
			move(cury, curx = x - maxx / 2);
			for (x2 = 0; x2 < maxx; curx++) {
				if (curx >= 0 && curx <= S_X)
					if (inch() == ' ')
						addch(array[y2][x2]);
					else
						move(cury, curx);
						/* one to right */
				x2++;
			}
		}
		y2++;
	}

}
