/*
 * Galactic Bloodshed, copyright (c) 1989 by Robert P. Chansky, 
 * smq@ucscb.ucsc.edu, mods by people in GB_copyright.h, enroll.dat.
 * Restrictions in GB_copyright.h.
 *  dosector.c 
 *  produce() -- produce, stuff like that, on a sector.
 *  spread()  -- spread population around.
 *  explore() -- mark sector and surrounding sectors as having been explored.
 */ 

#include "GB_copyright.h"
#define EXTERN extern
#include "vars.h"
#include "races.h"
#include "ships.h"
#include "doturn.h"
#include <math.h>

		/* produce stuff in sector */
produce(planet,s)
reg planettype *planet;
reg sectortype *s;
{
reg int fac;
int ss;
int maxsup;
int pfuel, pdes, pres;

reg float factor;
reg int eff,resdep;
reg struct plinfo *pinf;
reg int new;

if(s->resource < 0)
  s->resource = -s->resource;

	 check(planet,20);
	 factor = .4 * log10(1.0+(double)s->eff) * s->resource
	     * races[s->owner-1]->metabolism;
	 pinf = &planet->info[s->owner-1];


         pfuel = round_rand( factor * FUEL_PRODUCTION
		* (100 - s->mobilization) * ((s->des==DES_GAS)+1) );
	 prod_fuel[s->owner-1] += pfuel;

	 pres = round_rand( factor * RESOURCE_PRODUCTION 
			* (100 - s->mobilization) );
	 prod_res[s->owner-1] += pres;

	   pdes = round_rand( factor * DEST_PRODUCTION
			* s->mobilization );
	 prod_destruct[s->owner-1] += pdes;

		/* increase mobilization to planetary quota */
	 if (s->mobilization < pinf->mob_set) {
		if (pinf->resource + prod_res[s->owner-1] > 0) {
			s->mobilization++;
			prod_res[s->owner-1] -= round_rand(MOB_COST);
			prod_mob++;
	 	}
		check(planet,202);
	 } else if (s->mobilization > pinf->mob_set) {
		s->mobilization--;
		prod_mob--;
		check(planet,203);
	 }



	  /* add mobilization to get average mobilization */
	 avg_mob[s->owner-1] += s->mobilization;


	  /* do efficiency */
	 if (s->eff == 0)
		s->eff = 1;
	 else if (s->eff<100)
		s->eff += 
			round_rand(races[s->owner-1]->metabolism * float_rand());
	 else
		s->des = DES_PLATED;

check(planet,204);

	 if (s->is_wasted)
		s->popn = round_rand(s->popn * 0.10);
	 else if (s->popn >= races[s->owner-1]->number_sexes) {

	maxsup=maxsupport(races[s->owner-1],s,
		Compat[s->owner-1],planet->conditions[TOXIC]);

	ss = round_rand( ((float)maxsup-(float)s->popn) * races[s->owner-1]->birthrate);

           s->popn = MAX(0, MIN(MAX_SECT_POPN,s->popn + ss)); 
			/* keep the result nonnegative and below MAX */

	 } else {
		s->popn = round_rand(s->popn * .097);
	 }


	 if (s->popn == 0) 
		s->owner=0;

	 check(planet,21);
}


/*
 * spread population to surrounding sects. 
 *
 *	the way people move depends on the population of the planet unless 
 *	NO_SLIDING_SCALE_AUTOMOVE is set (again for reasons of computer 
 *	power).  Otherwise, the populace will automatically migrate to all 
 *	available surrounding areas if the planetary population is less than
 *	POPN_MOVE_SCALE_1, to either north-south or east-west areas if the
 *	population is less than POPN_MOVE_SCALE_2, and to only one adjacent
 *	random sector if it is greater.
 */


spread(pl,s,x,y)
reg planettype *pl;
reg sectortype *s;
reg int x,y;
{
int peep;

if(s->owner == 0) return;  /* gvc hack sometimes owned sectors
			      have no owner - causes bus error ! 
			      (someone has to figure this out) */

	 check(pl,22);
if (pl->is_sheep || (pl->slaved_to!=0 && pl->slaved_to!=s->owner))
	return;		/* no one wants to go anywhere */

   /* peep == number of people who want to move - at least one family wants to move*/

 peep = MAX(MIN(races[s->owner-1]->number_sexes, s->popn), 
		round_rand((float)s->popn * races[s->owner-1]->adventurism));

if (peep) {
 /*if (pl->type==TYPE_GASGIANT && s->des==DES_GAS && !int_rand(0,20)) {
	 /* random 'wind' sweeps people in this sector, somewhere else */
	  reg int x2,y2;
	  sectortype *s2,s3;
	/* huh.  For some reason, this seems to mess up the next planet's
	   sector map. */
	/*x2 = int_rand(0,pl->Maxx-1);
	y2 = int_rand(1,pl->Maxy-2);
	printf(" wind: %d,%d to %d,%d\n",x,y,x2,y2);
	s2 = &Sector(*pl,x2,y2);
	if (s2->des==DES_GAS) {
		s3 = *s2; 
		*s2 = *s; 
		*s = s3;
	}*/
 /*} else {*/

#if NO_SLIDING_SCALE_AUTOMOVE
 if (pl->popn < POPN_MOVE_SCALE_1) {
#endif
	Migrate2(pl,x+1,y,s,peep/4);
	Migrate2(pl,x-1,y,s,peep/4);
	Migrate2(pl,x,y+1,s,peep/4);
	Migrate2(pl,x,y-1,s,peep/4);
	 check(pl,23);
#if NO_SLIDING_SCALE_AUTOMOVE
 }
 else if (pl->popn < POPN_MOVE_SCALE_2) {
   x2 = int_rand(-1,1);
   y2 = (!x2) ? rposneg() : 0;	 /* 1/3 chance of migrating n-s*/
	Migrate2(pl,x+x2,y+y2,s,peep/2);   
	Migrate2(pl,x-x2,y-y2,s,peep/2);
			/* migrate to destination, from source */
 } else {
	Migrate2(pl, x + rposneg(), y + rposneg(), s, peep);
 }

#endif
 /*}*/
}

}



Migrate2(planet,xd,yd,ps,people)
planettype *planet;
reg int xd,yd;
sectortype *ps;
reg int people;
{
reg sectortype *pd;

	/* attempt to migrate beyond screen, or too many people */
 if (ps->popn <= people || (yd>=planet->Maxy) || (yd<0) ) 
	return;	

 if (xd<0) 
	xd=planet->Maxx-1;
 else if (xd>=planet->Maxx) 
	xd=0;

 pd = &Sector(*planet,xd,yd);

   {

	/* check for planetary compatibility */
   if (pd->is_wasted ||
   	people < races[ps->owner-1]->number_sexes || 
	float_rand()*100.0 > Compat[ps->owner-1] * races[ps->owner-1]->likes[pd->des]) {
	   return;

   }


 	if (!pd->owner) {
/*		printf("claiming %d,%d (%d): %d\n",xd,yd,ps->owner,people);*/
        	pd->popn += people;
        	ps->popn -= people;
		pd->owner = ps->owner;
		tot_captured++;
		if (Sectinfo[xd][yd].done)
			planet->info[pd->owner-1].numsectsowned++;
		Claims = 1;

 	} else if (pd->owner==ps->owner) {
        	pd->popn += people;
        	ps->popn -= people;

	} else if (isset(races[ps->owner-1]->atwar, pd->owner)) {
			/* we are at war with them; do land combat */
		reg float ofac,dfac;

		ofac = (float)MAX_SECT_POPN/(ps->popn+people) * 
			(ps->mobilization+1) *
			planet->info[ps->owner-1].comread/10000 *
			races[ps->owner-1]->fighters/10
			+ races[ps->owner-1]->tech/500.0;
/*		printf(" %d,%d (before MIN) ofac %.2f -- ",xd,yd,ofac);*/
		ofac = MIN(ofac, 1.0);
		dfac = (float)MAX_SECT_POPN/pd->popn * 
			(pd->mobilization+1) *
			planet->info[pd->owner-1].comread/10000 *
			races[pd->owner-1]->fighters/10
			+ races[pd->owner-1]->tech/500;
/*		printf("  dfac %.2f",dfac);*/
		dfac = MIN(dfac, 1.0);
		{reg int od,os;
		od=pd->popn;os=ps->popn;
		pd->popn *= 1. - ofac;
		ps->popn *= 1. - dfac;
/*		printf("def cas %d (of %d), off cas %d (of %d)\n",
			od-pd->popn,od,os-ps->popn,os); */
		}
		if (pd->popn <= 0) {
			    /* the invasion was successful */
			if (races[ps->owner-1]->Thing) {
				 /* mesos absorb the bodies of enemies */
				pd->popn += MIN(people, ps->popn);
				ps->popn -= MIN(people, ps->popn);
			} else {
				pd->popn = MIN(people, ps->popn);
				ps->popn -= MIN(people, ps->popn);
			}
			sects_lost[pd->owner-1]++;
			sects_gained[ps->owner-1]++;
			pd->owner = ps->owner;
			pd->eff *= 0.75;
			pd->mobilization *= 0.75;
			tot_captured++;
/*			printf(" sec %d,%d CAPTURED\n",xd,yd); */
		} else 
		   pd->popn -= round_rand(dfac);

 	} else {
		/* exchange prisoners */
		if (races[pd->owner-1]->captured_prisoners[ps->owner-1]<100)
			races[pd->owner-1]->captured_prisoners[ps->owner-1]++;
		if (races[ps->owner-1]->captured_prisoners[pd->owner-1]<100)
			races[ps->owner-1]->captured_prisoners[pd->owner-1]++;
	}
   }

	 check(planet,23);
}


/* mark sectors on the planet as having been "explored." for sea exploration
	on earthtype planets.  */

explore(planet, s, x, y, p)
reg planettype *planet;
reg sectortype *s;
reg int x,y,p;
{
reg int d;

	/* explore sectors surrounding sectors currently explored. */
 if (Sectinfo[x][y].explored) {
    Sectinfo[mod(x-1,planet->Maxx,d)][y].explored = p;
    Sectinfo[mod(x+1,planet->Maxx,d)][y].explored = p;
    if (y==0) {
	Sectinfo[x][1].explored = p;
    } else if (y==planet->Maxy-1) {
	Sectinfo[x][y-1].explored = p;
    } else {
	Sectinfo[x][y-1].explored = Sectinfo[x][y+1].explored = p;
    }
 
 } else if (s->owner==p)
	Sectinfo[x][y].explored = p;

}
