/*
 * Galactic Bloodshed, copyright (c) 1989 by Robert P. Chansky, 
 * smq@ucscb.ucsc.edu, mods by people in GB_copyright.h.
 * Restrictions in GB_copyright.h.
 *
 *  disk input/output routines & msc stuff
 *    all read routines lock the data they just accessed (the file is not
 *    closed).  write routines close and thus unlock that area.
 *
 *  openstardata(&fd),openpdata(&fd),openshdata(&fd),opensectdata(&fd)
 *     		-- open different files, return file descriptors
 *  getsdata(fd,&Sdata) -- get root dir for univ
 *  getrace(**racetype,racenum) -- get race from disk
 *  getstar(fd,**startype,starnum) -- gets star from disk
 *  getplanet(fd,**planettype,filepos) -- gets planet from disk
 *  getsector(fd,**sectortype,filepos) -- gets 1 sect from disk
 *  getsmap(fd,*sectortype,planet) -- gets sector map from disk
 *  getship(fd,**shiptype,shipnum) -- gets ship from disk
 *  putsdata(fd,&Sdata) -- put root dir for univ
 *  putrace(*racetype) -- put race to disk
 *  putstar(fd,*startype,starnum) -- put star to disk
 *  putplanet(fd,*planettype,filepos) -- put planet to disk
 *  putsector(fd,*sectortype,filepos) -- write sector to disk
 *  putsmap(fd,*sectortype,*planet) -- put smap to disk
 *  putship(fd,*shiptype,shipnum) -- put ship to disk
 *  int Numships(fd) -- return highest ship # from file
 *  getdeadship(fd, &filepos) -- filepos of next dead ship from shipfreedatafile
 *  makeshipdead(shipnum) -- kill a ship
 *  int Getracenum(char *) -- return race # from login list
 *  int Numraces() -- return # of races that exist right now
 *  Putpower() -- puts Power struct to disk for power command
 */

#include "GB_copyright.h"
#define EXTERN extern
#include "vars.h"
#include "ships.h"
#include "races.h"
#include "power.h"
#include <strings.h>
#include <sys/stat.h>
#include <signal.h>
#include <errno.h>
extern int errno;
int sys_nerr;
extern char *sys_errlist[];


openstardata(fd)
int *fd;
{
 /*printf(" openstardata\n");*/
 if ( (*fd = open(STARDATAFL, O_RDWR, 0777)) < 0) {
	  perror("openstardata");
	  printf("unable to open %s\n",STARDATAFL);
	  exit(-1);
 }
}


openshdata(fd)
int *fd;
{
if ( (*fd = open(SHIPDATAFL, O_RDWR, 0777)) < 0) {
	  perror("openshdata");
	  printf("unable to open %s\n",SHIPDATAFL);
	  exit(-1);
 }
}


openpdata(fd)
int *fd;
{
if ( (*fd = open(PLANETDATAFL, O_RDWR, 0777)) < 0) {
	  perror("openpdata");
	  printf("unable to open %s\n",PLANETDATAFL);
	  exit(-1);
 }
}


opensectdata(fd)
int *fd;
{
 if ( (*fd = open(SECTORDATAFL, O_RDWR, 0777)) < 0) {
	  perror("opensectdata");
	  printf("unable to open %s\n",SECTORDATAFL);
	  exit(-1);
 }
}

openracedata(fd)
int *fd;
{
 if ( (*fd = open(RACEDATAFL, O_RDWR, 0777)) < 0) {
	  perror("openrdata");
	  printf("unable to open %s\n",RACEDATAFL);
	  exit(-1);
 }
}


getsdata(fd,S)
int fd;
struct stardata *S;
{
  Fileread(fd,(char *)S, sizeof(struct stardata), STARDATAFL, 0);
}


getrace(fd, r,rnum)
int fd;
racetype **r;
int rnum;
{
 *r = (racetype *)malloc(sizeof(racetype));
 Fileread(fd, (char *)*r, sizeof(racetype), RACEDATAFL,
			(rnum-1)*sizeof(racetype) );
}

getstar(fd,s,star)
int fd;
startype **s;
int star;
{
 *s = (startype *)malloc(sizeof(startype));
 Fileread(fd,(char *)*s, sizeof(startype), STARDATAFL, 
	(int)(sizeof(Sdata)+star*sizeof(startype)) );
}


getplanet(fd,p,filepos)
int fd;
planettype **p;
int filepos;
{
 *p = (planettype *)malloc(sizeof(planettype));
 Fileread(fd,(char *)*p, sizeof(planettype), PLANETDATAFL, filepos );
}


getsector(fd,s,filepos)
int fd;
sectortype **s;
int filepos;
{
 *s = (sectortype *)malloc(sizeof(sectortype));
 Fileread(fd,(char *)*s, sizeof(sectortype), SECTORDATAFL, filepos);
}


getsmap(fd,map,p)
int fd;
sectortype *map;
planettype *p;
{
 Fileread(fd,(char *)map, p->Maxx * p->Maxy * sizeof(sectortype), 
	SECTORDATAFL, p->sectormappos);
}


int getship(fd,s,shipnum)
int fd;
shiptype **s;
int shipnum;
{
 struct stat buf;

 if ((*s = (shiptype *)malloc(sizeof(shiptype)))==NULL)
		printf("getship:Malloc() error \n"), exit();

 fstat(fd,&buf);
 if (buf.st_size / sizeof(shiptype) < shipnum) {
	(*s)->is_alive = 0;
	return 0;
 } else {

 	Fileread(fd, (char *)*s, sizeof(shiptype), SHIPDATAFL,
					(shipnum-1)*sizeof(shiptype) );
	return 1;
 }
}

/* gets the ship # listed in the top of the file SHIPFREEDATAFL. this
** might have no other uses besides build().
**  also locks the fd in shipdata.
*/
int getdeadship()
{
 struct stat buf;
 short shnum;
 int fd,lerr;
 int abort;

  if ( (fd = open(SHIPFREEDATAFL, O_RDWR, 0777)) < 0) {
  	perror("getdeadship");
  	printf("unable to open %s\n",SHIPFREEDATAFL);
  	exit(-1);
  }
 abort = 1;
 fstat(fd,&buf);

 if (buf.st_size && (abort ==1)) {
		/* put topmost entry in fpos */
	Fileread(fd, (char *)&shnum, sizeof(short), SHIPFREEDATAFL, 
			buf.st_size - sizeof(short) );
		/* erase that entry, since it will now be filled */
	ftruncate(fd, (long)(buf.st_size-sizeof(short)) );
	close_file(fd);
	return (int)shnum;
 } else
close_file(fd);
	return -1;
}



putsdata(fd,S)
int fd;
struct stardata *S;
{
  Filewrite(fd,(char *)S, sizeof(struct stardata), STARDATAFL, 0 );
}


putrace(fd, r)
int fd;
racetype *r;
{
 
 Filewrite(fd,(char *)r, sizeof(racetype), RACEDATAFL, 
		(r->Playernum-1)*sizeof(racetype) );
  }


putstar(fd,s,snum)
int fd;
startype *s;
int snum;
{
  Filewrite(fd,(char *)s, sizeof(startype), STARDATAFL, 
		(int)(sizeof(Sdata)+snum*sizeof(startype)) );
}

putplanet(fd,p,filepos)
int fd;
planettype *p;
int filepos;
{
 Filewrite(fd,(char *)p, sizeof(planettype), PLANETDATAFL, filepos );
}

putsector(fd,s,filepos)
int fd;
sectortype *s;
int filepos;
{
 Filewrite(fd,(char *)s, sizeof(sectortype), SECTORDATAFL, filepos );
}


putsmap(fd,map,p)
int fd;
sectortype *map;
planettype *p;
{
 Filewrite(fd, (char *)map,
	p->Maxx * p->Maxy * sizeof(sectortype), SECTORDATAFL, p->sectormappos);
}

putship(fd,s,shipnum)
int fd;
shiptype *s;
int shipnum;
{
 Filewrite(fd,(char *)s, sizeof(shiptype), SHIPDATAFL, 
		(shipnum-1)*sizeof(shiptype) );
}


int Numraces()
{
 struct stat buf;

 stat(RACEDATAFL,&buf);
 return( (int)(buf.st_size / sizeof(racetype)) );
}



int Numships(fd)	 /* return number of ships */
int fd;
{
 struct stat buf;

fstat(fd,&buf);

/* printf("numships %d\n",(int)(buf.st_size / sizeof(shiptype)) ); */
return( (int)(buf.st_size / sizeof(shiptype)) );
}




/* 
** writes the ship to the dead ship file at its end.
*/
makeshipdead(shipnum)
int shipnum;
{
 int fd;
 us shipno;
 struct stat buf;

  shipno = shipnum;	/* conv to u_short */

 if(shipno == 0)return;

  if ( (fd = open(SHIPFREEDATAFL, O_WRONLY, 0777)) < 0) {
	printf("fd = %d \n",fd);
	printf("errno = %d \n",errno);
	  perror("openshfdata");
	  printf("unable to open %s\n",SHIPFREEDATAFL);
	  exit(-1);
  }

  printf("ship #%u destroyed.\n", shipno);
	/* write the ship # at the very end of SHIPFREEDATAFL */
  fstat(fd,&buf);
  Filewrite(fd,(char *)&shipno, sizeof(shipno), SHIPFREEDATAFL, buf.st_size );
	close_file(fd);
}

Putpower(p)
struct power p[MAXPLAYERS];
{
int power_fd;

 if ( (power_fd = open(POWFL, O_RDWR, 0777)) < 0) {
	  perror("open power data");
	  printf("unable to open %s\n",POWFL);
	return;
 }
 write(power_fd, (char *)p, sizeof(*p)*MAXPLAYERS );
 close_file(power_fd);
}

Getpower(p)
struct power p[MAXPLAYERS];
{
int power_fd;

  if ( (power_fd = open(POWFL, O_RDONLY, 0777)) < 0) {
	  perror("open power data");
	  printf("unable to open %s\n",POWFL);
	return;  
} else {
	  read(power_fd, (char *)p, sizeof(*p)*MAXPLAYERS);
	  close_file(power_fd);
	}
}

Putblock(b)
struct block b[MAXPLAYERS];
{
int block_fd;

 if ( (block_fd = open(BLOCKDATAFL, O_RDWR, 0777)) < 0) {
	  perror("open block data");
	  printf("unable to open %s\n", BLOCKDATAFL);
	return;
 }
 write(block_fd, (char *)b, sizeof(*b)*MAXPLAYERS );
 close_file(block_fd);
}

Getblock(b)
struct block b[MAXPLAYERS];
{
int block_fd;

  if ( (block_fd = open(BLOCKDATAFL, O_RDONLY, 0777)) < 0) {
	  perror("open block data");
	  printf("unable to open %s\n", BLOCKDATAFL);
	return;  
} else {
	  read(block_fd, (char *)b, sizeof(*b)*MAXPLAYERS);
	  close_file(block_fd);
	}
}

